@@ -153,6 +153,9 @@ Style.MS_Induction = 'o'  # Marker style for induction scatter plots
 Style.wedgeAngle_deg = 25  # Angular size of wedge diagrams in degrees
 Style.LW_wedge = 0.125  # Linewidth in pt for minor boundaries in wedge diagrams
 Style.LW_wedgeMajor = 0.375  # Linewidth in pt for major layer boundaries in wedge diagrams
+Style.TS_ticks = 12  # Text size in pt for tick marks on radius scale
+Style.TS_desc = 14  # Text size in pt for model description and label
+Style.TS_super = 16  # Text size in pt for overall ("suptitle") label with multiple wedges
 
 # Complex dipole plots
 Style.MW_dip = {'synodic': 5, 'orbital': 6, 'true anomaly': 6, 'synodic harmonic': 4.5}  # Marker size for each period in complex dipole plots
@@ -256,7 +259,7 @@ FigMisc.tMagLbl = None  # List of strings to use to describe the times listed in
 FigMisc.tMagEval_s = spice.str2et('2000-01-01T11:58:55.816')  # Time in seconds past J2000 at which to evaluate magnetic field surface maps. spiceypy.str2et will convert a datetime string to the correct format. Accepts an array.
 FigMisc.LARGE_ADJUST = True  # Whether to make certain labels better for cramped spaces, including removing colorbars (True is more pared down, and overrides nLon/LatTicks below.)
 FigMisc.BASYM_WITH_SYM = True  # Whether to plot Basym plot and Bsym plot on the same figure
-FigMisc.vCompMagSurf = 'mag'  # Component to use for induced field surface strength plots. Options are ['x', 'y', 'z', 'mag'].
+FigMisc.vCompMagSurf = 'mag'  # Component to use for induced field surface strength plots. Options are ['x', 'y', 'z', 'mag', 'all'].
 FigMisc.nPPGCmapRes = 180  # Number of points per great circle to use for map angular resolution
 FigMisc.DO_360 = False  # Whether to range longitudes from 0 to 360 or from -180 to +180
 FigMisc.nLatTicks = 7  # Number of ticks to mark on latitude axis
