@@ -159,7 +159,16 @@ def IceLayers(Planet, Params):
     # a whole-layer conductive profile. The densities will change slightly, so we depart from self-consistency
     # here. Repeated applications of IceConvect will get closer to self-consistency.
 
-    if Planet.Pb_MPa > 0:
+    if Planet.Pb_MPa > 0 and Planet.Pb_MPa < Planet.P_MPa[0]:
+        negDeltaPmsg = f'Calculated Pb value of {Planet.Pb_MPa:.2f} MPa is less than surface pressure of {Planet.P_MPa[0]:.2f} MPa. ' + \
+            'This likely means Tb_K is set too high. Try to decrease and run again to get a valid model.'
+        if Params.ALLOW_BROKEN_MODELS:
+            log.warning(negDeltaPmsg + ' ALLOW_BROKEN_MODELS is True, so execution will continue.')
+            Planet.Do.VALID = False
+        else:
+            raise ValueError(negDeltaPmsg)
+
+    elif Planet.Pb_MPa > 0:
         if Planet.Do.CLATHRATE:
             """ For ice shells insulated by a layer of clathrate at the surface or against the bottom
                 Calculates state variables of the layer with each pressure step
@@ -465,6 +474,11 @@ def OceanLayers(Planet, Params):
 
         for i in range(iStart, Planet.Steps.nOceanMax):
             Planet.phase[Planet.Steps.nSurfIce+i] = Planet.Ocean.EOS.fn_phase(POcean_MPa[i], TOcean_K[i]).astype(np.int_)
+            if i < 4 and Planet.phase[Planet.Steps.nSurfIce+i] != 0:
+                log.debug(f'Top ocean layers (i={i}) are not liquid. This will cause indexing problems. ' +
+                          'T will be set to exceed the melting temp temporarily to construct at least 4 ocean layers.')
+                TOcean_K[i] = GetTfreeze(Planet.Ocean.EOS, POcean_MPa[i], TOcean_K[i]) + Planet.Ocean.TfreezeOffset_K
+                Planet.phase[Planet.Steps.nSurfIce+i] = 0
             log.debug(f'il: {Planet.Steps.nSurfIce+i:d}; P_MPa: {POcean_MPa[i]:.3f}; ' +
                       f'T_K: {TOcean_K[i]:.3f}; phase: {Planet.phase[Planet.Steps.nSurfIce+i]:d}')
             if Planet.phase[Planet.Steps.nSurfIce+i] < 2:
