@@ -27,6 +27,17 @@ def SetupInit(Planet, Params):
     if Planet.Do.TAUP_SEISMIC: CheckCompat('obspy')  # TauP (accessed as obspy.taup)
     if Params.CALC_NEW_INDUCT: CheckCompat('MoonMag')  # MoonMag
 
+    # Afford for additional MoI lower-bound uncertainty under non-hydrostatic conditions of 3% of C/MR^2,
+    # in accordance with Gao and Stevenson (2013): https://doi.org/10.1016/j.icarus.2013.07.034
+    if Planet.Bulk.CuncertaintyUpper is None:
+        Planet.Bulk.CuncertaintyUpper = Planet.Bulk.Cuncertainty
+    if Planet.Bulk.CuncertaintyLower is None:
+        if Planet.Do.NONHYDROSTATIC:
+            Planet.Bulk.CuncertaintyLower = Planet.Bulk.Cuncertainty + 0.03 * Planet.Bulk.Cmeasured
+        else:
+            Planet.Bulk.CuncertaintyLower = Planet.Bulk.Cuncertainty
+    Planet = SetCMR2strings(Planet)
+
     # Waterless bodies. We have to do this first, before filename
     # generation, to ensure ocean comp is set.
     if Planet.Do.NO_H2O:
@@ -47,6 +58,7 @@ def SetupInit(Planet, Params):
         Planet.Steps.nSurfIce = 0
         Planet.Steps.nOceanMax = 1
         Planet.Steps.nHydroMax = 1
+        Planet.Steps.iSilStart = 0
         Planet.Ocean.comp = 'none'
         Planet.Ocean.wOcean_ppt = 0.0
         Planet.Ocean.deltaP = 0.1
@@ -58,7 +70,7 @@ def SetupInit(Planet, Params):
             Planet.Ocean.EOS = GetOceanEOS('none', None, np.linspace(0, 1, 4), np.linspace(0, 1, 4), None)
 
     # Get filenames for saving/loading
-    Params.DataFiles, Params.FigureFiles = SetupFilenames(Planet, Params)
+    Planet, Params.DataFiles, Params.FigureFiles = SetupFilenames(Planet, Params)
 
     # Set steps and settings for unused options to zero, check that we have settings we need
     # Core settings
@@ -317,23 +329,26 @@ def SetupFilenames(Planet, Params, exploreAppend=None, figExploreAppend=None):
         saveLabel += f'NoH2O_qSurf{Planet.Bulk.qSurf_Wm2*1e3:.1f}mWm2'
         setStr = f'$q_\mathrm{{surf}}\,{Planet.Bulk.qSurf_Wm2*FigLbl.qMult:.1f}\,\si{{{FigLbl.fluxUnits}}}$'
         label += setStr
+        Planet.compStr = r'No~\ce{H2O}'
     else:
         if Planet.Ocean.comp == 'PureH2O':
             saveLabel += f'{Planet.Ocean.comp}_Tb{Planet.Bulk.Tb_K}K'
             setStr = f'Pure \ce{{H2O}}'
             label += f'{setStr}, $T_b\,\SI{{{Planet.Bulk.Tb_K}}}{{K}}$'
+            Planet.compStr = r'Pure~\ce{H2O}'
         else:
             saveLabel += f'{Planet.Ocean.comp}_{Planet.Ocean.wOcean_ppt:.1f}ppt' + \
                         f'_Tb{Planet.Bulk.Tb_K}K'
             setStr = f'${Planet.Ocean.wOcean_ppt*FigLbl.wMult:.1f}\,\si{{{FigLbl.wUnits}}}$ \ce{{{Planet.Ocean.comp}}}'
             label += setStr + \
                 f', $T_b\,\SI{{{Planet.Bulk.Tb_K}}}{{K}}$'
+            Planet.compStr = f'${Planet.Ocean.wOcean_ppt*FigLbl.wMult:.1f}\,\si{{{FigLbl.wUnits}}}$~\ce{{{Planet.Ocean.comp}}}'
         if Planet.Do.CLATHRATE:
             saveLabel += '_Clathrates'
             label += ' w/clath'
         if Planet.Do.POROUS_ICE:
             saveLabel += '_PorousIce'
-            label += ' w/$\phi_{ice}$'
+            label += ' w/$\phi_\mathrm{ice}$'
         if Planet.Do.PORE_EOS_DIFFERENT:
             if Planet.Sil.poreComp == 'PureH2O':
                 saveLabel += f'_{Planet.Sil.poreComp}Pores'
@@ -341,10 +356,13 @@ def SetupFilenames(Planet, Params, exploreAppend=None, figExploreAppend=None):
             else:
                 saveLabel += f'_{Planet.Sil.poreComp}_{Planet.Sil.wPore_ppt:.1f}pptPores'
                 label += f', {Planet.Sil.wPore_ppt*FigLbl.wMult:.1f}\,\si{{{FigLbl.wUnits}}} \ce{{{Planet.Sil.poreComp}}} pores'
+        elif Planet.Do.POROUS_ROCK:
+            saveLabel += '_PorousRock'
+            label += ' w/$\phi_\mathrm{sil}$'
     if Planet.Sil.mantleEOSName is not None: saveLabel += f'_{Planet.Sil.mantleEOSname}'
 
     Planet.saveLabel = saveLabel
-    Planet.tradeLabel = f'{label}, $C/MR^2\,{Planet.Bulk.Cmeasured}\pm{Planet.Bulk.Cuncertainty}$'
+    Planet.tradeLabel = f'{label}, $C/MR^2\,{Planet.CMR2str}$'
     Planet.label = label
     if Params.DO_INDUCTOGRAM:
         inductBase = f'{Planet.name}_{Params.Induct.inductOtype}'
@@ -356,7 +374,7 @@ def SetupFilenames(Planet, Params, exploreAppend=None, figExploreAppend=None):
                                        comp=Planet.Ocean.comp, inductBase=inductBase,
                                        exploreAppend=figExploreAppend)
 
-    return DataFiles, FigureFiles
+    return Planet, DataFiles, FigureFiles
 
 
 def SetupLayers(Planet):
@@ -382,3 +400,33 @@ def SetupLayers(Planet):
     Planet.P_MPa[0] = Planet.Bulk.Psurf_MPa  # Set first layer to surface pressure
 
     return Planet
+
+
+def SetCMR2strings(Planet):
+    # Create strings to describe input MoI in terminal outputs and plots/tables
+    if Planet.Bulk.CuncertaintyLower == Planet.Bulk.CuncertaintyUpper:
+        Planet.CMR2strPrint = f'{Planet.Bulk.Cmeasured:.4f} ± {Planet.Bulk.CuncertaintyLower:.4f}'
+        Planet.CMR2str = f'{Planet.Bulk.Cmeasured}\pm{Planet.Bulk.CuncertaintyLower}'
+        Planet.CMR2str5 = f'{Planet.Bulk.Cmeasured:.5f}\pm{Planet.Bulk.CuncertaintyLower:.5f}'
+    else:
+        Planet.CMR2strPrint = f'{Planet.Bulk.Cmeasured:.4f} + {Planet.Bulk.CuncertaintyUpper:.4f} - {Planet.Bulk.CuncertaintyLower:.4f}'
+        if round(Planet.Bulk.Cmeasured, 3) == Planet.Bulk.Cmeasured and round(Planet.Bulk.CuncertaintyUpper, 3) == Planet.Bulk.CuncertaintyUpper:
+            CmidStr = f'{Planet.Bulk.Cmeasured:.3f}'
+            CpStr = f'+{Planet.Bulk.CuncertaintyUpper:.3f}'
+            CmStr = f'-{Planet.Bulk.CuncertaintyLower:.3f}'
+        elif round(Planet.Bulk.Cmeasured, 4) == Planet.Bulk.Cmeasured and round(Planet.Bulk.CuncertaintyUpper, 4) == Planet.Bulk.CuncertaintyUpper:
+            CmidStr = f'{Planet.Bulk.Cmeasured:.4f}'
+            CpStr = f'+{Planet.Bulk.CuncertaintyUpper:.4f}'
+            CmStr = f'-{Planet.Bulk.CuncertaintyLower:.4f}'
+        else:
+            CmidStr = f'{Planet.Bulk.Cmeasured:.5f}'
+            CpStr = f'+{Planet.Bulk.CuncertaintyUpper:.5f}'
+            CmStr = f'-{Planet.Bulk.CuncertaintyLower:.5f}'
+        if FigMisc.TEX_INSTALLED:
+            Planet.CMR2str = f'{CmidStr}\substack{{{CpStr} \\\\ {CmStr}}}'
+            Planet.CMR2str5 = f'{Planet.Bulk.Cmeasured:.5f}\substack{{+{Planet.Bulk.CuncertaintyUpper:.5f} \\\\ -{Planet.Bulk.CuncertaintyLower:.5f}}}'
+        else:
+            Planet.CMR2str = f'{CmidStr}^{{{CpStr}}}_{{{CmStr}}}'
+            Planet.CMR2str5 = f'{Planet.Bulk.Cmeasured:.5f}^{{+{Planet.Bulk.CuncertaintyUpper:.5f}}}_{{-{Planet.Bulk.CuncertaintyLower:.5f}}}'
+
+    return Planet
