@@ -5,12 +5,13 @@ Overridden by any settings contained within PPBody.py files.
 import os
 from PlanetProfile.Utilities.defineStructs import ParamsStruct, ExploreParamsStruct, Constants
 
-configVersion = 8  # Integer number for config file version. Increment when new settings are added to the default config file.
+configVersion = 9  # Integer number for config file version. Increment when new settings are added to the default config file.
 
 Params = ParamsStruct()
 Params.VERBOSE =       False  # Provides extra runtime messages. Overrides QUIET below
 Params.QUIET =         False  # Hides all log messages except warnings and errors
 Params.QUIET_MOONMAG = True  # If True, sets MoonMag logging level to WARNING, otherwise uses the same as PlanetProfile.
+Params.QUIET_LBF = True  # If True, sets lbftd and mlbspline logging levels to ERROR, otherwise uses the same as PlanetProfile.
 Params.printFmt = '[%(levelname)s] %(message)s'  # Format for printing log messages
 # The below flags allow or prevents extrapolation of EOS functions beyond the definition grid.
 Params.EXTRAP_ICE = {'Ih':False, 'II':False, 'III':False, 'V':False, 'VI':False, 'Clath':False}
@@ -26,7 +27,7 @@ Params.CALC_NEW_INDUCT =  True  # Recalculate magnetic induction responses?
 Params.CALC_NEW_ASYM =    False  # Recalculate asymmetric boundary plot(s)?
 Params.CALC_SEISMIC =     True  # Calculate sound speeds and elastic moduli?
 Params.CALC_CONDUCT =     True  # Calculate electrical conductivity?
-Params.CALC_ASYM =        False  # Calculate asymmetric shape?
+Params.CALC_ASYM =        True  # Calculate induction with asymmetric shape?
 Params.RUN_ALL_PROFILES = False  # Whether to run all PPBody.py files for the named body and plot together
 Params.SPEC_FILE =        False  # Whether we are running a specific file or files
 Params.COMPARE =          False  # Whether to plot each new run against other runs from the same body
@@ -98,13 +99,16 @@ Params.spiceBSP = {
     'Jupiter': 'jup365.bsp',  # Generic kernel for Jupiter + Galilean moons
     'Saturn': 'sat427.bsp',  # Generic kernel for Saturn + large moons
     'Uranus': 'ura111.bsp',  # Generic kernel for Uranus + large moons
-    'Neptune': 'nep095.bsp'  # Generic kernel for Neptune + large moons
+    'Neptune': 'nep097.bsp'  # Generic kernel for Neptune + large moons
 }
 
 Params.SCmagFnameFmt = {
     'Galileo': 'ORB*_SYS3.TAB',
     'Cassini': '*_KRTP_1S.TAB',
-    'Juno': 'fgm_jno_l3_*pc_r1s_v*.sts'
+    'Juno': 'fgm_jno_l3_*pc_r1s_v*.sts',
+    'Voyager 1': 'vg1_*_sph.tab',
+    'Voyager 2': 'vg2_*_sph.tab',
+    'Clipper': 'mag_clp*.tab'
 }
 Params.SCnames = list(Params.SCmagFnameFmt.keys())
 Params.spiceSC = {scName: os.path.join(Params.spiceDir, scName) for scName in Params.SCnames}
